require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

module OpenvasCli
  describe VasSchedule do
    def valid_params
      {
        :name => "Test_#{Time.now.strftime('%Y-%m-%d_%H:%M:%S')}", 
        :comment => "Test Schedule [Automatically Generated]", 
        :first_time => Time.now + 86400,
        :period => VasPeriod.new(:number => 1, :period => :day)
      }
    end
    
    def get_t_string(t)
      t.strftime("%Y-%m-%d_%H:%M")
    end
    
    after(:each) do
      all = VasSchedule.get_all
      all.each { |s| s.destroy! if s.name =~ /Test_.*/i }
    end

    it 'should pull all available schedules' do
      schedules = VasSchedule.get_all
      schedules.should_not be nil
      schedules.each{ |s| s.should be_valid }
    end
    
    it 'should pull a schedule by ID' do  
      all = VasSchedule.get_all
      all.should_not be nil

      unless all.empty?    
        schedule_id = all.choice.id
        schedule = VasSchedule.get_by_id(schedule_id)
        schedule.should_not be nil
      end
    end

    it 'should return nil if bogus ID is provided' do
      lambda {
        VasSchedule.get_by_id("8675309").should be nil
      }.should_not raise_error
    end
    
    it 'default sort should be by name' do
      all = VasSchedule.get_all
      
      old_val = ""
      all.each { |s|
        s.name.should >= old_val
        old_val = s.name
      }
    end
    
    it 'should sort by schedule_id' do
      all = VasSchedule.get_all(:sort_by=>:schedule_id)
      
      old_val = ""
      all.each { |s|
        s.id.should >= old_val
        old_val = s.id
      }
    end
    
    it 'should sort_by first_time' do
      all = VasSchedule.get_all(:sort_by=>:first_time)
      
      old_val = Time.at(0)
      all.each { |s|
        s.first_time.should >= old_val
        old_val = s.first_time
      }
    end
    
    it 'should sort_by next_time' do
      all = VasSchedule.get_all(:sort_by=>:next_time)
      
      old_val = Time.at(0)
      all.each { |s|
        s.next_time.should >= old_val
        old_val = s.next_time
      }
    end
    
    it 'should create a new schedule' do
      s = VasSchedule.new(valid_params)
      s.should be_valid
      s.save!
      
      s.should_not be_changed
      s.id.should_not be nil
      
      new_s = VasSchedule.get_by_id(s.id)
      new_s.should_not be nil
      new_s.id.should            == s.id
      new_s.name.should          == s.name
      new_s.comment.should       == s.comment
      get_t_string(new_s.first_time).should    == get_t_string(s.first_time)
      new_s.period.should        == s.period
    end
      
    it 'should modify an existing schedule' do
      s = VasSchedule.new(valid_params)
      s.save!
      
      old_id = s.id
      
      s.comment = "NEW COMMENT"
      s.save!
      
      s.id.should_not == old_id
      
      VasSchedule.get_by_id(old_id).should be nil
      
      new_s = VasSchedule.get_by_id(s.id)
      new_s.should_not be nil
      new_s.id.should            == s.id
      new_s.name.should          == s.name
      new_s.comment.should       == s.comment
      get_t_string(new_s.first_time).should    == get_t_string(s.first_time)
      new_s.period.should        == s.period
    end
    
    it 'should report changed when name is changed' do
      s = VasSchedule.new(valid_params)
      s.should_not be_changed
      
      s.name = "NEW NAME"
      s.should be_changed
    end
    
    it 'should report changed when comment is changed' do
      s = VasSchedule.new(valid_params)
      s.should_not be_changed
      
      s.comment = "NEW COMMENT"
      s.should be_changed
    end
    
    it 'should report changed when first_time is changed' do
      s = VasSchedule.new(valid_params)
      s.should_not be_changed
      
      s.first_time = s.first_time
      s.should_not be_changed
      
      s.first_time = Time.now + 30
      s.should be_changed
    end
    
    it 'should report changed when period is changed' do
      s = VasSchedule.new(valid_params)
      s.should_not be_changed
      
      s.period = s.period
      s.should_not be_changed
      
      s.period = s.period + VasPeriod.new(:number => 2, :period => :day)
      s.should be_changed
    end
  end
end
