require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')
require 'ipaddress'

module OpenvasCli
  describe VasTarget do
    before(:each) do
      @valid_chars = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a',
                     'b', 'c', 'd', 'e', 'f', 'A', 'B', 'C', 'D', 'E', 'F', '-']
    end
    
    after(:each) do
      all = VasTarget.get_all
      
      all.each { |t|
        t.destroy! if t.name =~ /Test_.*/i
      }
    end

    def valid_params
      {
        :name=>"Test_#{Time.now.strftime('%Y-%m-%d_%H:%M:%S')}",
        :hosts => VasTarget.get_local_subnets
      }
    end
    
    it 'should get all available targets' do
      targs = VasTarget.get_all
      targs.count.should > 0
      
      targs.each { |t|
        t.should be_valid
      }
    end
    
    it 'should require a non-empty vas_id' do
      t = VasTarget.new(valid_params)
      t.id = ""
      
      t.should_not be_valid
    end
    
    it 'should require a valid UUID format' do
      t = VasTarget.new(valid_params)
      t.id = "This is not a valid UUID"
      t.should_not be_valid
      t.should have(1).errors
      
      1.upto(10) {
        uuid = ''
        1.upto(rand(40) + 1) {
          uuid += @valid_chars[rand(@valid_chars.length)]
        }
        t.id = uuid
        t.should be_valid
      }
    end
    
    it 'should generate a valid list of subnets' do
      VasTarget.get_local_subnets.each { |sn|
        net = IPAddress(sn)
        net.should be_network
      }
    end
    
    it 'should create a new target if the id is empty' do
      t = VasTarget.new(valid_params)
      t.id = nil
      t.save!
      
      t.id.should_not be nil
      
      n_t = VasTarget.get_by_id(t.id)
      n_t.should_not be nil
      n_t.id.should == t.id
      n_t.name.should == t.name
      n_t.should have(t.hosts.count).hosts
      n_t.hosts.each { |h|
        t.hosts.should include h
      }
    end
    
    it 'should record a change when the name has changed' do
      t = VasTarget.new(valid_params)
      
      t.should_not be_changed
      
      t.name = "NEW NAME"
      t.should be_changed
    end
    
    it 'should record a change when the comment has changed' do
      t = VasTarget.new(valid_params)
      t.should_not be_changed
      
      t.comment = "NEW COMMENT"
      t.should be_changed
    end
    
    it 'should record a change when the hosts have been re-assigned' do
      t = VasTarget.new(valid_params)
      t.should_not be_changed
      
      t.hosts = ['10.10.10.0/24', '192.168.1.0/24']
      t.should be_changed
    end
    
    it 'should record a change when a host has been added' do
      t = VasTarget.new(valid_params)
      t.should_not be_changed
      
      t.hosts << "10.10.10.0/24"
      t.should be_changed
    end
    
    it 'should record a change when a host has been removed' do
      t = VasTarget.new(valid_params.merge({:hosts => ['10.10.10.0/24', '192.168.1.0/24']}))
      t.should be_valid
      t.should have(2).hosts
      
      t.hosts.delete_if { |h| h == '192.168.1.0/24'}
      t.should have(1).hosts
      
      t.should be_changed
    end
    
    it 'should create a new target and delete the old one when something has changed.' do
      t = VasTarget.new(valid_params)
      t.save!
      org_id = t.id
      
      t.hosts = ['10.10.10.0/24', '192.168.1.0/24']
      t.save!
      
      t.id.should_not == org_id
      
      srch = VasTarget.get_by_id(org_id)
      srch.should be nil
    end
    
    it 'should show a change when a new credential is added' do
      t = VasTarget.new(valid_params)
      t.credentials[:ssh] = VasLscCredential.new(:name => "Test_login", 
                                                 :login => "myLogin",
                                                 :comment => "Should never be saved", 
                                                 :password => "reallyBadPassword")
                                                 
      t.should be_changed
    end
    
    it 'should pull hosts_string' do
      t = VasTarget.new(valid_params)
      t.hosts << ['10.10.10.0/24', '192.168.1.0/24']
      
      t.hosts_string.should == (VasTarget.get_local_subnets + ['10.10.10.0/24', '192.168.1.0/24']).join(", ")
    end
    
    it 'should parse hosts_string' do
      my_hosts = ['10.10.10.0/24', '192.168.1.0/24']
      
      t = VasTarget.new(valid_params)
      t.hosts_string = my_hosts.join(", ")
      
      t.hosts.each { |h|
        my_hosts.include?(h).should be true
      }
    end
    
    it 'update_attributes should update hosts when hosts_string is passed' do
      t = VasTarget.new(valid_params)
      org_hosts_string = t.hosts_string
      new_hosts = ['10.10.10.0/24', '192.168.1.0/24']
      t.update_attributes({:hosts_string => new_hosts.join(", ")})
      
      t.hosts_string.should_not == org_hosts_string
      t.hosts.each { |h|
        new_hosts.include?(h).should be true
      }
    end
    
    it 'should report the progress of a task appropriately' do
      
    end
  end
end
