require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

module OpenvasCli
  describe VasTask do
    after(:each) do
      all = VasTask.get_all
      all.each { |t| t.destroy! if t.name =~ /Test_.*/i }
      
      all = VasTarget.get_all
      all.each { |t| t.destroy! if t.name =~ /Target_Test_.*/i }
    end
    
    def time_stamp
      Time.now.strftime('%Y-%m-%d_%H:%M:%S')
    end
    
    def valid_params
      {
        :name => "Test_#{time_stamp}",
        :config_id => get_config_id, 
        :target_id => get_target_id, 
        :schedule => VasSchedule.new(schedule_params)
      }
    end
    
    def schedule_params
      {
        :name => "Test_#{time_stamp}", 
        :comment => "Test Schedule [Automatically Generated]", 
        :first_time => Time.now + 86400,
        :period => VasPeriod.new(:number => 1, :period => :day)
      }
    end
    
    def get_config_id
      VasConfig.get_all.choice.id
    end
    
    def get_target_id
      targs = VasTarget.get_all
      if targs.count == 0 
        t = VasTarget.create!(valid_target_params)
      else
        t = targs.choice
      end
      
      t.id
    end
    
    def valid_target_params
      {
        :name  => "Test_Target_#{time_stamp}",
        :hosts => VasTarget.get_local_subnets
      }
    end
    
    def get_gateway
      `route -n`.split("\n").each { |rline|
        return rline.split(/\s+/)[1] if rline =~ /\A\s*0\.0\.0\.0/
      }
    end
    
    it 'should create a valid task' do
      t = VasTask.new(valid_params)
      t.should be_valid
    end

    it 'should pull all tasks' do
      tasks = VasTask.get_all
      
      tasks.should_not be nil
      
      tasks.each { |t| 
        t.should be_valid 
      }
    end
    
    it 'should create a task' do
      t = VasTask.new(valid_params)
      t.save.should_not be nil
      
      t.id.should_not be nil
      t.id.should_not be_empty
      
      new_t = VasTask.get_by_id(t.id)
      new_t.should_not be nil
      
      new_t.id.should == t.id      
      new_t.name.should == t.name
      new_t.config_id.should == t.config_id
      new_t.target_id.should == t.target_id
      
      new_t.destroy!
    end
    
    it 'should pull a single task given the task_id' do
      all = VasTask.get_all
      
      if all.length > 0
        task = all[0]
      
        new_task = VasTask.get_by_id(task.id)
        
        task.id.should == new_task.id
      end
    end
    
    it 'should start a new task and run it to the end', :slow => true do
      config = nil
      VasConfig.get_all.each { |cfg|
        config = cfg if cfg.name =~ /full and fast/i
      }
      config.should_not be nil
      
      target = VasTarget.new(
        :name  => "Target_Test_Gateway_#{time_stamp}",
        :hosts => [get_gateway]
      )
      target.save!
      
      task = VasTask.new(
        :name      => "Test_Task_Runner_#{time_stamp}",
        :comment   => "Going to run this one.",
        :config_id => config.id,
        :target_id => target.id
      )
      task.save!
      
      task.start
      
      
      tsk = VasTask.get_by_id(task.id)
      count = 0
      while tsk.status =~ /requested/i && count < 360
        sleep 1
        tsk = VasTask.get_by_id(task.id)
        count += 1
      end
      
      count.should be < 360
            
      tsk.status.should =~ /running/i
      
      overall = 0
      max = -99999
      while tsk.status =~ /running/i do
        sleep 3 
        
        tsk = VasTask.get_by_id(task.id)
        tsk.progress.overall.should >= overall unless tsk.progress.overall == -1
        overall = tsk.progress.overall
        max = overall if overall > max
      end
      
      max.should > 0
    end
    
    it 'should stop a task completely', :slow => true do
      config = nil
      VasConfig.get_all.each { |cfg|
        config = cfg if cfg.name = "Full and fast"
      }
      config.should_not be nil
      
      target = VasTarget.new(
        :name  => "Target_Test_Gateway_#{time_stamp}",
        :hosts => [get_gateway]
      )
      target.save!
      
      task = VasTask.new(
        :name      => "Test_Task_Runner_#{time_stamp}",
        :comment   => "Going to run this one.",
        :config_id => config.id,
        :target_id => target.id
      )
      task.save!
      
      task.start
      
      
      tsk = VasTask.get_by_id(task.id)
      count = 0
      while tsk.status =~ /requested/i && count < 360
        sleep 1
        tsk = VasTask.get_by_id(task.id)
        count += 1
      end
      
      count.should be < 360
            
      tsk.status.should =~ /running/i
      
      sleep 30
      tsk.stop
      sleep 60
      tsk = VasTask.get_by_id(task.id)
      tsk.status.should =~ /stopped/i
            
    end
    
    it 'should pause a task and restart it' do 
      config = nil
      VasConfig.get_all.each { |cfg|
        config = cfg if cfg.name = "Full and fast"
      }
      config.should_not be nil
      
      target = VasTarget.new(
        :name  => "Target_Test_Gateway_#{time_stamp}",
        :hosts => [get_gateway]
      )
      target.save!
      
      task = VasTask.new(
        :name      => "Test_Task_Runner_#{time_stamp}",
        :comment   => "Going to run this one.",
        :config_id => config.id,
        :target_id => target.id
      )
      task.save!
      
      task.start
      
      
      tsk = VasTask.get_by_id(task.id)
      count = 0
      while tsk.status =~ /requested/i && count < 360
        sleep 1
        tsk = VasTask.get_by_id(task.id)
        count += 1
      end
      
      count.should be < 360
            
      tsk.status.should =~ /running/i
      
      sleep 30
    end
  end
end
